package com.qmth.themis.business.util;

import com.qmth.themis.business.constant.SystemConstant;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @Description: redis util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/4/15
 */
@Component
public class RedisUtil {

    @Resource
    RedisTemplate redisTemplate;

    /**
     * 获取机构信息
     *
     * @param orgId
     * @return
     */
    public Object getOrg(Long orgId) {
        return redisTemplate.opsForValue().get(SystemConstant.ORG + orgId);
    }

    /**
     * 设置机构信息
     *
     * @param orgId
     * @param o
     */
    public void setOrg(Long orgId, Object o) {
        redisTemplate.opsForValue().set(SystemConstant.ORG + orgId, o, SystemConstant.REDIS_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 删除机构缓存
     *
     * @param orgId
     */
    public void deleteOrg(Long orgId) {
        redisTemplate.delete(SystemConstant.ORG + orgId);
    }

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    public Object getUser(Long userId) {
        return redisTemplate.opsForValue().get(SystemConstant.USER + userId);
    }

    /**
     * 获取用户会话信息
     *
     * @param sessionId
     * @return
     */
    public Object getUserSession(String sessionId) {
        return redisTemplate.opsForValue().get(SystemConstant.SESSION + sessionId);
    }

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    public void deleteUser(Long userId) {
        redisTemplate.delete(SystemConstant.USER + userId);
    }

    /**
     * 删除用户会话缓存
     *
     * @param sessionId
     */
    public void deleteUserSession(String sessionId) {
        redisTemplate.delete(SystemConstant.SESSION + sessionId);
    }

    /**
     * 设置用户信息
     *
     * @param userId
     * @param o
     */
    public void setUser(Long userId, Object o) {
        redisTemplate.opsForValue().set(SystemConstant.USER + userId, o, SystemConstant.REDIS_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 刷新用户缓存
     *
     * @param userId
     */
    public void refreshUser(Long userId) {
        redisTemplate.expire(SystemConstant.USER + userId, SystemConstant.REDIS_REFRESH_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 获取用户缓存过期时间
     *
     * @param userId
     * @return
     */
    public Long getUserExpire(Long userId) {
        return redisTemplate.getExpire(SystemConstant.USER + userId, TimeUnit.SECONDS);
    }

    /**
     * 设置用户session信息
     *
     * @param sessionId
     * @param o
     */
    public void setUserSession(String sessionId, Object o) {
        redisTemplate.opsForValue().set(SystemConstant.SESSION + sessionId, o, SystemConstant.REDIS_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 刷新用户session缓存
     *
     * @param sessionId
     */
    public void refreshUserSession(String sessionId) {
        redisTemplate.expire(SystemConstant.SESSION + sessionId, SystemConstant.REDIS_REFRESH_EXPIRE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 获取用户session缓存过期时间
     *
     * @param sessionId
     * @return
     */
    public Long getUserSessionExpire(String sessionId) {
        return redisTemplate.getExpire(SystemConstant.SESSION + sessionId, TimeUnit.SECONDS);
    }

    /**
     * 批量获取key的value
     *
     * @param keys
     * @return
     */
    public List<?> multiGet(Set keys) {
        return redisTemplate.opsForValue().multiGet(keys);
    }

    /**
     * 获取key like
     *
     * @param key
     * @return
     */
    public Set getKeyPatterns(String key) {
        if (null != key) {
            return redisTemplate.keys(key + "*");
        } else {
            return null;
        }
    }

    /**
     * 设置hash
     *
     * @param key
     * @param hashKey
     * @param hashValue
     */
    public void setSessionTopicList(String key, String hashKey, Object hashValue) {
        redisTemplate.opsForHash().put(key, hashKey, hashValue);
    }

    /**
     * 获取hash
     *
     * @param key
     * @param hashKey
     * @return
     */
    public Object getSessionTopicList(String key, String hashKey) {
        return redisTemplate.opsForHash().get(key, hashKey);
    }

    /**
     * hash删除
     *
     * @param key
     * @param hashKey
     */
    public void deleteSessionTopicList(String key, String hashKey) {
        redisTemplate.opsForHash().delete(key, hashKey);
    }

    /**
     * 获取hash大小
     *
     * @param key
     * @return
     */
    public Long getHashSize(String key) {
        return redisTemplate.opsForHash().size(key);
    }

    /**
     * 获取hash map
     *
     * @param key
     * @return
     */
    public Map getHashEntries(String key) {
        return redisTemplate.opsForHash().entries(key);
    }
}